/*
 * APA102.c
 *
 * Created: 17.11.2016 19:04:30
 *  Author: Thomas
 */ 

#include <avr/io.h>
#include <avr/interrupt.h>
#include "../BasicTypes.h"

#include "APA102.h"


volatile uint8_t UpdateLEDsIsAktive;
APA102Frame_T APA102Frame;

static uint8_t* volatile ISR_LEDAddress;


#ifdef __AVR_ATmega328PB__

void APA102Init(void)
{
	#define UBRR_VALUE (F_CPU / (2*500000)) - 1
	#define UCPHA10 UCSZ10
	
	DDRB |= (1<<DDB5); // XCK1 Master Mode
	
	// Set MSPI mode of operation and SPI data mode 3
	UCSR1C = (3<<UMSEL10)|(1<<UCPHA10)|(1<<UCPOL1);
	
	// Enable transmitter
	UCSR1B = (1<<TXEN1);
	
	// Set baud rate. IMPORTANT: After enabled
	UBRR1 = UBRR_VALUE;
}


void APA102UpdateLEDs(void)
{
	if (UpdateLEDsIsAktive)
	{
		return;
	}
	
	UpdateLEDsIsAktive = TRUE;
	
	ISR_LEDAddress = (uint8_t*)&APA102Frame;
	
	UCSR1B |= (1<<UDRIE1);
}
 
 
ISR(USART1_UDRE_vect, ISR_BLOCK)
{
	uint8_t* tmpISR_LEDAddress = ISR_LEDAddress;
	
	UDR1 = *tmpISR_LEDAddress;

	ISR_LEDAddress = ++tmpISR_LEDAddress;
		
	if (tmpISR_LEDAddress >= ((uint8_t*)&APA102Frame + sizeof(APA102Frame)))
	{
		UCSR1B &= ~(1<<UDRIE1);
		UpdateLEDsIsAktive = FALSE;
	}
}

#endif


#ifdef __AVR_ATmega328P__

void APA102Init(void)
{
	// Set SS, MOSI and SCK output
	DDRB |= (1<<DDB2)|(1<<DDB3)|(1<<DDB5);
	
	//Enable SPI, Master, Mode3 and set clock rate FCK/32 (625kHz@20MHz)
	SPCR = (1<<SPE)|(1<<MSTR)|(1<<CPHA)|(1<<CPOL)|(2<<SPR0);
	
	// Double SPI Speed
	SPSR = (1<<SPI2X);
}


void APA102UpdateLEDs(void)
{
	if (UpdateLEDsIsAktive)
	{
		return;
	}
	
	UpdateLEDsIsAktive = TRUE;
	
	uint8_t* tmpISR_LEDAddress = (uint8_t*)&APA102Frame;
	
	ISR_LEDAddress = tmpISR_LEDAddress + 1;
	
	SPSR |= (1<<SPIF);
	SPCR |= (1<<SPIE);
	
	SPDR = *tmpISR_LEDAddress;
}


ISR(SPI_STC_vect, ISR_BLOCK)
{
	uint8_t* tmpISR_LEDAddress = ISR_LEDAddress;
	
	SPDR = *tmpISR_LEDAddress;

	ISR_LEDAddress = ++tmpISR_LEDAddress;
	
	if (tmpISR_LEDAddress >= ((uint8_t*)&APA102Frame + sizeof(APA102Frame)))
	{
		SPCR &= ~(1<<SPIE);
		UpdateLEDsIsAktive = FALSE;
	}
}

#endif