/*
 * Analog.c
 *
 * Created: 25.11.2016 18:48:15
 *  Author: Thomas
 */ 

#include <avr/io.h>
#include "../BasicTypes.h"

#include "Analog.h"
#include "../main.h"


uint8_t AnalogPotiHSV_H;
uint8_t AnalogPotiHSV_S;
uint8_t AnalogPotiHSV_V;


typedef enum tagChannelSwitch {
	Channel_HSV_H,
	Channel_HSV_S,
	Channel_Brightness,
} ChannelSwitch_T;


static ChannelSwitch_T ChannelSwitch;
static uint8_t FirstConversion;


void AnalogInit()
{
	// ADC Multiplexer Selection Register
	// REFS1 REFS0 ADLAR MUX3 MUX2 MUX1 MUX0
	#define ADMUXREF (1<<REFS0)	// REFS = AVCC
	ADMUX = ADMUXREF | (ChannelPotiHSV_H<<MUX0);
	
	// ADCSRA  ADC Control and Status Register A
	// ADEN ADSC ADATE ADIF ADIE ADPS2 ADPS1 ADPS0	
	ADCSRA = (1<<ADEN)|(1<<ADSC)|(7<<ADPS0);		//On + start + Division Factor 128 / 156kHZ@20MHz
	
	ChannelSwitch = Channel_HSV_H;
	FirstConversion = TRUE;
}


void AnalogReadChannels()
{
	static uint32_t SumAnalog;
	static uint8_t Conversions;
	static uint16_t AnalogInputOldPotiHSV_H;
	static uint16_t AnalogInputOldPotiHSV_S;
	static uint16_t AnalogInputOldPotiBrightness;


	if (ADCSRA & (1<<ADSC))
	{
		return;
	}
	
	uint16_t tmpAnalogInput = ADC;

	if (FirstConversion)
	{
		FirstConversion = FALSE;
	}
	else
	{
		SumAnalog += tmpAnalogInput;
	
		if (++Conversions == 16)
		{
			Conversions = 0;
	
			uint16_t tmpAnalogAverage = (uint16_t)(SumAnalog >> 4);
			SumAnalog = 0;
	
			switch (ChannelSwitch)
			{
				case Channel_HSV_H:
				{
					if ((tmpAnalogAverage >= (AnalogInputOldPotiHSV_H + 4)) ||
						 ((tmpAnalogAverage + 4) <= AnalogInputOldPotiHSV_H))
					{
						AnalogInputOldPotiHSV_H = tmpAnalogAverage & ~3;
			
						AnalogPotiHSV_H = AnalogInputOldPotiHSV_H >> 2;
					}
					
					ADMUX = ADMUXREF | (ChannelPotiHSV_S<<MUX0);
					ChannelSwitch = Channel_HSV_S;
				}
				break;


				case Channel_HSV_S:
				{
					if ((tmpAnalogAverage >= (AnalogInputOldPotiHSV_S + 4)) ||
						((tmpAnalogAverage + 4) <= AnalogInputOldPotiHSV_S))
					{
						AnalogInputOldPotiHSV_S = tmpAnalogAverage & ~3;
						
						AnalogPotiHSV_S = AnalogInputOldPotiHSV_S >> 2;
					}
					
					ADMUX = ADMUXREF | (ChannelPotiHSV_V<<MUX0);
					ChannelSwitch = Channel_Brightness;
				}
				break;


				case Channel_Brightness:
				{
					if ((tmpAnalogAverage >= (AnalogInputOldPotiBrightness + 4)) ||
						((tmpAnalogAverage + 4) <= AnalogInputOldPotiBrightness))
					{
						AnalogInputOldPotiBrightness = tmpAnalogAverage & ~3;
						
						AnalogPotiHSV_V = (AnalogInputOldPotiBrightness >> 2);
					}
					
					ADMUX = ADMUXREF | (ChannelPotiHSV_H<<MUX0);
					ChannelSwitch = Channel_HSV_H;
DEBUG_TOGGLE_PIN3					
				}
				break;

			} // Switch
			
			FirstConversion = TRUE;
			
		} // Average
		
	} // FirstConversion	

	ADCSRA |= (1<<ADSC);	// Start new sampling
}

