
/*
 * HSV2RGB.S
 *
 * Created: 19.11.2016 10:21:38
 *  Author: Thomas
 */ 


 #include <avr/io.h>
 
// r0, r18r27, r30, r31 mssen nicht restauriert werden
// r24/25 HSV struct
// r22/23 LED struct



// ***************************************************
// **		HSV zu RGB Farbwerte konvertieren		**
// ***************************************************
// **	IN:	h,s,v	=	Reg. r18,r19,r20			**
// **  OUT: R,G,B	=	Reg. r18,r19,r20			**
// **	RI:	Register Intern r0+r1 +r21 +r26-r29 + Z	**
// ***************************************************

// ***************************************************
.global HSV2RGB
HSV2RGB:
			push	r28
			push	r29

			movw	Z,r24
			ldd		r18, Z+0
			ldd		r19, Z+1
			ldd		r20, Z+2

			call	HSV2RGBI

			// 0 Rot, 1 Grn, 2 Blau
			movw	Z,r22
			std		Z+0, r18
			std		Z+1, r19
			std		Z+2, r20

			pop		r29
			pop		r28

			eor	r1, r1
			ret
// ***************************************************

// ***************************************************
.global HSV2GRB
HSV2GRB:
			push	r28
			push	r29

			movw	Z,r24
			ldd		r18, Z+0
			ldd		r19, Z+1
			ldd		r20, Z+2

			call	HSV2RGBI

			// 0 Grn, 1 Rot, 2 Blau
			movw	Z,r22
			std		Z+1, r18
			std		Z+0, r19
			std		Z+2, r20

			pop		r29
			pop		r28

			eor	r1, r1
			ret
// ***************************************************

// ***************************************************
.global HSV2BGR
HSV2BGR:
			push	r28
			push	r29

			movw	Z,r24
			ldd		r18, Z+0
			ldd		r19, Z+1
			ldd		r20, Z+2

			call	HSV2RGBI

			// Byte 0 Blau, 1 Grn, 2 Rot
			movw	Z,r22
			std		Z+2, r18
			std		Z+1, r19
			std		Z+0, r20

			pop		r29
			pop		r28

			eor	r1, r1
			ret
// ***************************************************


// ***************************************************
.global HSV2APA102cGBGR
HSV2APA102cGBGR:
			push	r28
			push	r29

			movw	Z,r24
			ldd		r18, Z+0
			ldd		r19, Z+1
			ldd		r20, Z+2

			call	HSV2RGBI

			// Byte 0 Global, 1 Blau, 2 Grn, 3 Rot
			movw	Z,r22
			std		Z+3, r18	
			std		Z+2, r19	 
			std		Z+1, r20	
								 
			pop		r29
			pop		r28

			eor	r1, r1
			ret
// ***************************************************


// Register Definitionen
rP = 26		; p Ergebnis
rQ = 27		; q Ergebnis
//rT = 28	; t Ergebnis

rPos = 29	; Position in der Region

rHR = 18	; IN: h	/ OUT: R
rSG = 19	; IN: s / OUT: G
rVB = 20	; IN: v / OUT: B
	
rtmp = 21	; Temp


// ****		Macros Berechnungen p, q, t

// p = (hsv.v * (255 - hsv.s)) >> 8
.macro	mHSV_Calc_P
			mov		rtmp,rSG
			com		rtmp
			mul		rtmp,rVB
			mov		rP,r1
.endm

// q = (hsv.v * (255 - ((hsv.s * Position) >> 8))) >> 8
.macro	mHSV_Calc_Q
			mul		rPos,rSG
			com		r1
			mul		r1,rVB
			;mov		rQ,r1
.endm

// t = (hsv.v * (255 - ((hsv.s * (255 - Position)) >> 8))) >> 8
.macro	mHSV_Calc_T
			com		rPos
			mul		rPos,rSG
			com		r1
			mul		r1,rVB
			;mov		rT,r1
.endm
// ***************************************************


// ****		HSV nach RGB konvertieren

			// R = G = B = v (Grautne)
HSVGRAU:	mov		rHR,rVB
			mov		rSG,rVB
			;mov		rVB,rVB
			ret

			// Einstiegspunkt 
HSV2RGBI:	tst		rSG
			breq	HSVGRAU		; Wenn s = 0 dann Grautne
			
			// ijmp Adresse Region und Position ber Tabelle ermitteln
HSV0000:	ldi		rtmp,3
			mul		rtmp,rHR
			movw	Z,r0
			subi	ZL,lo8(-(HSV_tab))
			sbci	ZH,hi8(-(HSV_tab)) 

			lpm		rtmp,Z+		; ijmp Region LOW
			push	rtmp
			lpm		rtmp,Z+		; ijmp Region HIGH
			push	rtmp
			lpm		rPos,Z		; Position in der Region
				
			// p wird in jeder Region bentigt
			mHSV_Calc_P

			// Sprung auf Region
			pop		ZH
			pop		ZL
			lsr		ZH
			ror		ZL 
			ijmp

			// Region 0
HSVSR0:		mHSV_Calc_T

			mov		rHR,rVB
			mov		rSG,r1			;mov	rSG,rT
			mov		rVB,rP
			ret

			// Region 1
HSVSR1:		mHSV_Calc_Q

			mov		rHR,r1			;mov	rHR,rQ
			mov		rSG,rVB
			mov		rVB,rP
			ret

			// Region 2
HSVSR2:		mHSV_Calc_T

			mov		rHR,rP
			mov		rSG,rVB
			mov		rVB,r1			;mov	rVB,rT
			ret

			// Region 3
HSVSR3:		mHSV_Calc_Q

			mov		rHR,rP
			mov		rSG,r1			;mov	rSG,rQ
			;mov		rVB,rVB
			ret

			// Region 4
HSVSR4:		mHSV_Calc_T

			mov		rHR,r1			;mov	rHR,rT
			mov		rSG,rP
			;mov		rVB,rVB
			ret

			// Region 5
HSVSR5:		mHSV_Calc_Q

			mov		rHR,rVB
			mov		rSG,rP
			mov		rVB,r1			;mov	rVB,rQ
			ret

// ****		Ende
// ***************************************************



// ***************************************************
// Tabelle fr h 0-255
// ***************************************************

 			// ijmp Adresse fr HSV_tab / Regionen definieren
			#define		iJR0	lo8(HSVSR0), hi8(HSVSR0)
			#define		iJR1	lo8(HSVSR1), hi8(HSVSR1)	
			#define		iJR2	lo8(HSVSR2), hi8(HSVSR2)	
			#define		iJR3	lo8(HSVSR3), hi8(HSVSR3)	
			#define		iJR4	lo8(HSVSR4), hi8(HSVSR4)	
			#define		iJR5	lo8(HSVSR5), hi8(HSVSR5)	

			// ijmp Adresse Region und Position in der Region
HSV_tab:	.BYTE iJR0,   0, iJR0,   6, iJR0,  12, iJR0,  18, iJR0,  24, iJR0,  30, iJR0,  36, iJR0,  42, iJR0,  48, iJR0,  54
			.BYTE iJR0,  60, iJR0,  66, iJR0,  72, iJR0,  78, iJR0,  84, iJR0,  90, iJR0,  96, iJR0, 102, iJR0, 108, iJR0, 114
			.BYTE iJR0, 120, iJR0, 126, iJR0, 132, iJR0, 138, iJR0, 144, iJR0, 150, iJR0, 156, iJR0, 162, iJR0, 168, iJR0, 174
			.BYTE iJR0, 180, iJR0, 186, iJR0, 192, iJR0, 198, iJR0, 204, iJR0, 210, iJR0, 216, iJR0, 222, iJR0, 228, iJR0, 234
			.BYTE iJR0, 240, iJR0, 246, iJR0, 252, iJR1,   2, iJR1,   8, iJR1,  14, iJR1,  20, iJR1,  26, iJR1,  32, iJR1,  38
			.BYTE iJR1,  44, iJR1,  50, iJR1,  56, iJR1,  62, iJR1,  68, iJR1,  74, iJR1,  80, iJR1,  86, iJR1,  92, iJR1,  98
			.BYTE iJR1, 104, iJR1, 110, iJR1, 116, iJR1, 122, iJR1, 128, iJR1, 134, iJR1, 140, iJR1, 146, iJR1, 152, iJR1, 158
			.BYTE iJR1, 164, iJR1, 170, iJR1, 176, iJR1, 182, iJR1, 188, iJR1, 194, iJR1, 200, iJR1, 206, iJR1, 212, iJR1, 218
			.BYTE iJR1, 224, iJR1, 230, iJR1, 236, iJR1, 242, iJR1, 248, iJR1, 254, iJR2,   4, iJR2,  10, iJR2,  16, iJR2,  22
			.BYTE iJR2,  28, iJR2,  34, iJR2,  40, iJR2,  46, iJR2,  52, iJR2,  58, iJR2,  64, iJR2,  70, iJR2,  76, iJR2,  82
			.BYTE iJR2,  88, iJR2,  94, iJR2, 100, iJR2, 106, iJR2, 112, iJR2, 118, iJR2, 124, iJR2, 130, iJR2, 136, iJR2, 142
			.BYTE iJR2, 148, iJR2, 154, iJR2, 160, iJR2, 166, iJR2, 172, iJR2, 178, iJR2, 184, iJR2, 190, iJR2, 196, iJR2, 202
			.BYTE iJR2, 208, iJR2, 214, iJR2, 220, iJR2, 226, iJR2, 232, iJR2, 238, iJR2, 244, iJR2, 250, iJR3,   0, iJR3,   6
			.BYTE iJR3,  12, iJR3,  18, iJR3,  24, iJR3,  30, iJR3,  36, iJR3,  42, iJR3,  48, iJR3,  54, iJR3,  60, iJR3,  66
			.BYTE iJR3,  72, iJR3,  78, iJR3,  84, iJR3,  90, iJR3,  96, iJR3, 102, iJR3, 108, iJR3, 114, iJR3, 120, iJR3, 126
			.BYTE iJR3, 132, iJR3, 138, iJR3, 144, iJR3, 150, iJR3, 156, iJR3, 162, iJR3, 168, iJR3, 174, iJR3, 180, iJR3, 186
			.BYTE iJR3, 192, iJR3, 198, iJR3, 204, iJR3, 210, iJR3, 216, iJR3, 222, iJR3, 228, iJR3, 234, iJR3, 240, iJR3, 246
			.BYTE iJR3, 252, iJR4,   2, iJR4,   8, iJR4,  14, iJR4,  20, iJR4,  26, iJR4,  32, iJR4,  38, iJR4,  44, iJR4,  50
			.BYTE iJR4,  56, iJR4,  62, iJR4,  68, iJR4,  74, iJR4,  80, iJR4,  86, iJR4,  92, iJR4,  98, iJR4, 104, iJR4, 110
			.BYTE iJR4, 116, iJR4, 122, iJR4, 128, iJR4, 134, iJR4, 140, iJR4, 146, iJR4, 152, iJR4, 158, iJR4, 164, iJR4, 170
			.BYTE iJR4, 176, iJR4, 182, iJR4, 188, iJR4, 194, iJR4, 200, iJR4, 206, iJR4, 212, iJR4, 218, iJR4, 224, iJR4, 230
			.BYTE iJR4, 236, iJR4, 242, iJR4, 248, iJR4, 254, iJR5,   4, iJR5,  10, iJR5,  16, iJR5,  22, iJR5,  28, iJR5,  34
			.BYTE iJR5,  40, iJR5,  46, iJR5,  52, iJR5,  58, iJR5,  64, iJR5,  70, iJR5,  76, iJR5,  82, iJR5,  88, iJR5,  94
			.BYTE iJR5, 100, iJR5, 106, iJR5, 112, iJR5, 118, iJR5, 124, iJR5, 130, iJR5, 136, iJR5, 142, iJR5, 148, iJR5, 154
			.BYTE iJR5, 160, iJR5, 166, iJR5, 172, iJR5, 178, iJR5, 184, iJR5, 190, iJR5, 196, iJR5, 202, iJR5, 208, iJR5, 214
			.BYTE iJR5, 220, iJR5, 226, iJR5, 232, iJR5, 238, iJR5, 244, iJR5, 250
.end


