/*
 * QuadratureDecoder.c
 *
 * Created: 26.11.2016 18:38:05
 *  Author: Thomas
 */ 

#include <avr/io.h>
#include <avr/pgmspace.h>

#include "QuadratureDecoder.h"


static const uint8_t QuadratureDecoderTab[] PROGMEM = {
	0x01, 0x02,	// 0 +1/-2
	0x03, 0x00,	// 1 +3/-0
	0x00, 0x03, // 2 +0/-3
	0x02, 0x01,	// 3 +2/-1
};


void QuadratureDecoderInit(uint8_t inputs, pQuadratureDec_T instance)
{
	instance->Internal.IndexQDTab = (inputs & 0x03);
}


uint8_t QuadratureEncoding(uint8_t inputs, pQuadratureDec_T instance)
{
	inputs &= 0x03;
	
	if (inputs == instance->Internal.IndexQDTab)
	{
		return 0;
	}
	
	uint16_t tmpCompareIncDec = pgm_read_word(&QuadratureDecoderTab[instance->Internal.IndexQDTab<<1]);
	uint8_t tmpCompareInc = (uint8_t)tmpCompareIncDec;
	uint8_t tmpCompareDec = tmpCompareIncDec >> 8;
	
	uint8_t tempQDPosition = instance->Counter;
	
	if (inputs == tmpCompareInc)
	{
		tempQDPosition++;
	}
	else if (inputs == tmpCompareDec)
	{
		tempQDPosition--;
	}
	else
	{
		return -1;
	}

	instance->Counter = tempQDPosition;
	
	instance->Internal.IndexQDTab = inputs;
	
	return 0;
}


int8_t QuadratureDecoderDiv4Step(pQuadratureDec_T instance)
{
	if ((instance->Counter >= (instance->Internal.CounterOld + 4)) ||
		((instance->Counter + 4) <= instance->Internal.CounterOld))
	{
			instance->Internal.CounterOld = instance->Counter;
			
			uint8_t tmpStep = instance->Counter - instance->Internal.StepOld;
			instance->Internal.StepOld = instance->Counter;
			
			return ((int8_t)tmpStep)>>2;
	}
	
	return 0;
}